﻿using API_LoL_Project.Mapper;
using API_LoL_Project.Middleware;
using DTO;
using Microsoft.AspNetCore.Mvc;
using Model;
using StubLib;
using System.Xml.Linq;

// For more information on enabling Web API for empty projects, visit https://go.microsoft.com/fwlink/?LinkID=397860

namespace API_LoL_Project.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class ChampionsController : ControllerBase
    {
        public IChampionsManager dataManager;
        private readonly ILogger<ChampionsController> _logger;

        /* public ChampionsController(IChampionsManager dataManager)
         {          
             this.dataManager = dataManager;
         }*/
        public ChampionsController(IDataManager dataManager, ILogger<ChampionsController> logger)
        {
            this.dataManager = dataManager.ChampionsMgr;
            this._logger = logger;
        }



        // GET: api/<ChampionController>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<ChampionDTO>>> Get([FromQuery] Request.PageRequest request)
        {
            try
            {
                var totalcount = await dataManager.GetNbItems();
                if (request.count + request.index > totalcount)
                {
                    _logger.LogWarning("No chamions found with Id");
                    return BadRequest("No chamions found with Id ");
                }
                _logger.LogInformation("Executing {Action} with parameters: {Parameters}", nameof(Get), request);;
                var champions = await dataManager.GetItems(request.index, request.count, request.orderingPropertyName, request.descending);
                IEnumerable<ChampionDTO> res = champions.Select(c => c.ToDTO());
                if (res.Count() <= 0 || res == null)
                {
                    _logger.LogWarning("No chamions found with Id");
                    return BadRequest("No chamions found with Id ");
                }
                return Ok(res);
            }
            catch (Exception e)
            {
                return BadRequest(e.Message);

            }


        }



        // GET api/<ChampionController>/5
        [HttpGet("{name}")]
        public async Task<ActionResult<ChampionDTO>> GetChampionsByName(string name)
        {
            try
            {

                var champion = await dataManager
                .GetItemsByName(name, 0, await dataManager.GetNbItems());
                _logger.LogInformation("Executing {Action} with name : {championName}", nameof(GetChampionsByName), name);
                ChampionDTO res = champion.First().ToDTO();
                if (res == null)
                {
                    _logger.LogWarning("No chamions found with {name}", name);  ;
                    return NotFound();
                }
                return Ok(res);

            }
            catch (Exception e)
            {
                return BadRequest(e.Message);

            }
        }


        // POST api/<ChampionController>
        [HttpPost]
        public async Task<IActionResult> Post([FromBody] ChampionDTO value)
        {
            try
            {
                var newChampion = value.ToModel();
                await dataManager.AddItem(newChampion);
                return CreatedAtAction(nameof(Get), newChampion) ;
            }
            catch (Exception e)
            {
                return BadRequest(e.Message);

            }

        }

        // PUT api/<ChampionController>/5
        [HttpPut("{name}")]
        public async Task<IActionResult> Put(string name, [FromBody] ChampionDTO value)
        {
            

            try
            {
                var champion = await dataManager
                .GetItemsByName(name, 0, await dataManager.GetNbItems());
                await dataManager.UpdateItem(champion.First(), value.ToModel());
                return Ok();
            }
            catch(Exception e)
            {
                return BadRequest(e.Message);
            }
        }

        // DELETE api/<ChampionController>/5
        [HttpDelete("{name}")]
        public async Task<IActionResult> Delete(string name)
        {
            try
            {
                var champion = await dataManager
                .GetItemsByName(name, 0, await dataManager.GetNbItems());
                if (champion != null) await dataManager.DeleteItem(champion.First());
                else
                {
                    return NotFound();
                }
                return Ok();
            }
            catch (Exception e)
            {
                return BadRequest(e.Message);
            }
            
        }

        /*   [HttpGet]
           public async Task<IActionResult> NbChampions()
           {
               var champions = await dataManager.GetItems(0, await dataManager.GetNbItems());
               IEnumerable<ChampionDTO> res = champions.Select(c => c.toDTO());
               return Ok(res);

           }*/

        [HttpGet("/{name}/skins")]
        public async Task<ActionResult<Skin>> GetChampionsSkins(string name)
        {
            var champions = await dataManager.GetItemsByName(name, 0, await dataManager.GetNbItems());
            //skinsDTO
            IEnumerable<Skin> res = champions.First().Skins;

            return Ok(res);
        }

        [HttpGet("/{name}/skills")]
        public async Task<ActionResult<Skin>> GetChampionsSkills(string name)
        {
            var champions = await dataManager.GetItemsByName(name, 0, await dataManager.GetNbItems());
            //SkillDTO
            IEnumerable<Skill> res = champions.First().Skills;

            return Ok(res);
        }

        /*[HttpGet("/{name}/skins")]
        public async Task<ActionResult<Skill> NbChampions()
        {
            var champions = await dataManager.GetItems(0, await dataManager.GetNbItems());
            IEnumerable<ChampionDTO> res = champions.Select(c => c.toDTO());
            return Ok(res);

        }*/
    }
}
